@@ -12,8 +12,6 @@ import subprocess
 import json
 from Bio import SeqIO, Phylo
 from packaging import version
-# from bohra.bohra_logger import logger
-# from bohra.utils.write_report import Report
 
 
 class RunSnpDetection(object):
@@ -38,7 +36,6 @@ class RunSnpDetection(object):
         # add the handlers to the logger
         self.logger.addHandler(ch)
         self.logger.addHandler(fh)
-
         # get date and time
         self.now = datetime.datetime.today().strftime("%d_%m_%y_%H")
         self.day = datetime.datetime.today().strftime("%d_%m_%y")
@@ -49,11 +46,14 @@ class RunSnpDetection(object):
         self.pipeline = args.pipeline
         self.preview = True if self.pipeline == 'preview' else False
         self.logger.info(f"You are running bohra in {self.pipeline} mode.")
+        # self.snippy_singularity = args.snippy_singularity
+        # self.abritamr_singularity = args.abritamr_singularity
+        self.job_id = self._name_exists(args.job_id)
+        self.logger.info(f"Job ID is set {self.job_id}")
         # path to reference and mask
-        if self.pipeline != 'a':
-            self.ref = pathlib.Path(args.reference)
-        else:
-            self.ref = ''
+        self.ref = pathlib.Path(args.reference)
+        self.logger.info(f"The reference is {self.ref}")
+        self.link_file(self.ref)
         # 
         # (args.mask)
         if args.mask:
@@ -69,9 +69,10 @@ class RunSnpDetection(object):
         # # path to a source file for tracking of reference, jobid and mask
         # self.source_log_path = pathlib.Path(self.workdir, 'source.log')
         # job id
-        self.job_id = self._name_exists(args.job_id)
-        self.logger.info(f"Job ID is set {self.job_id}")
+        
         self.check_rerun()
+        self.keep = True if args.keep == 'Y' else False
+        self.gubbins = args.gubbins 
         # other variables
         # min aln 
         self.minaln = args.minaln
@@ -88,11 +89,14 @@ class RunSnpDetection(object):
         self.user = getpass.getuser()
         
         self.gubbins = False
-        
+        self.use_singularity = args.use_singularity
         self.mdu = args.mdu
+        # self.logger.info(f"{self.mdu}")
         if isinstance(args.prefillpath, str):
             self.prefillpath = args.prefillpath
         elif self.mdu:
+            # self.use_singularity = True
+            # self.logger.info(f"You are running bohra on mdu business - so singularity containers will be used automatically {self.use_singularity}")
             self.prefillpath = f"{pathlib.Path('/', 'home', 'seq', 'MDU', 'QC')}/"
         else:
             self.prefillpath = ''
@@ -106,8 +110,6 @@ class RunSnpDetection(object):
         self.assembler = args.assembler
         self.snippy_version = ''
         self.assembler_dict = {'shovill': 'shovill', 'skesa':'skesa','spades':'spades.py'}
-        self.use_singularity = args.use_singularity
-        self.singularity_path = args.singularity_path
         self.set_snakemake_jobs()
 
     def check_queue(self, queue):
@@ -182,11 +184,12 @@ class RunSnpDetection(object):
         self.logger.info(f"Checking that snippy is installed and recording version.")
         version_pat = re.compile(r'\bv?(?P<major>[0-9]+)\.(?P<minor>[0-9]+)\.(?P<release>[0-9]+)(?:\.(?P<build>[0-9]+))?\b')
         try:
-            snippy = subprocess.run(['snippy', '--version'], stderr=subprocess.PIPE)
-            snippy = snippy.stderr.decode().strip()
-            self.snippy_version = version_pat.search(snippy)
-            self.logger.info(f"Snippy {snippy} found. Good job!")
-            return(version_pat.search(snippy))
+            snippy = subprocess.run(['snippy --version 2>&1'], capture_output=True, encoding = "utf-8", shell = True)
+            snippy = snippy.stdout
+            # print(snippy.strip())
+            # self.snippy_version = version_pat.search(snippy.strip())
+            # self.logger.info(f"Snippy {self.snippy_version} found. Good job!")
+            return(version_pat.search(snippy.strip()))
         except FileNotFoundError:
             self.logger.info(f"snippy is not installed.")
             raise SystemExit
@@ -233,15 +236,25 @@ class RunSnpDetection(object):
         
         self.check_installation(self.assembler_dict[self.assembler])
 
+    def check_abritamr(self):
+
+        if shutil.which('abritamr'):
+            self.logger.info(f"abritamr is installed")
+        elif shutil.which('abriTAMR'):
+            self.logger.info(f"abritamr is installed")
+        else:
+            self.logger.warning(f"abritamr is not installed, resistome will not be determined. Check https://github.com/MDU-PHL/abritamr for installation instructions.")
+
 
     def check_assemble_accesories(self):
         '''
         check the assembly accessories mlst, kraken, abricate and prokka
         '''
-        accessories = ['mlst', 'kraken2', 'abricate', 'prokka']
+        accessories = ['mlst', 'kraken2', 'prokka']
         
         for a in accessories:
             self.check_installation(a)
+        self.check_abritamr()
     
     def check_roary(self):
         '''
@@ -293,7 +306,72 @@ class RunSnpDetection(object):
         else:
             self.logger.warning(f"Your kraken DB is not installed in the expected path. Please re-read bohra installation instructions.")
             raise SystemExit
-            
+
+        
+    def run_with_gubbins(self):
+        '''
+        rename core and distance files
+        '''
+        if self.gubbins:
+            self.logger.info(f"You have chosen to run gubbins. Existing core files will be archived and not removed.")
+            corefiles = sorted(pathlib.Path(self.workdir, self.job_id).glob('core*'))
+            if corefiles:
+                for core in corefiles:
+                    new = f"{core}".replace('core', 'core_uncorrected')
+                    cmd = f"mv {core} {new}"
+                    subprocess.run(cmd,shell = True)
+            dists = pathlib.Path(self.workdir,self.job_id, 'distances.tab')
+            new_dists = f"{dists}".replace('distances', 'distances_uncorrected')
+            if dists.exists():
+                cmd = f"mv {dists} {new_dists}"
+                subprocess.run(cmd,shell = True)
+            self.keep = True
+    
+    def rerun_report(self):
+        '''
+        Remove report directory from previous run 
+        '''
+        # os.chdir(self.workdir)
+        
+        p1 = pathlib.Path(self.workdir, self.job_id, 'report')
+        p2 = pathlib.Path(self.workdir, self.job_id, f"report_{self.orignal_date}")
+        if self.keep:
+            self.logger.info(f"Archiving previous report files")
+            cmd = f"mv {p1} {p2}"
+            self.remove_core()
+        else:
+            self.logger.info("Removing previous report files.")
+            cmd = f"if [ -d {p1} ];then rm -r {p1}; fi"
+        subprocess.run(cmd, shell = True)
+    
+    def remove_core(self):
+        '''
+        Need to remove core_isolates.txt to get snakemake to redo snippy core step
+        '''
+        self.logger.info(f"Removing previous snippy-core output.")
+        corefiles = sorted(pathlib.Path(self.workdir, self.job_id).glob('core*'))
+        if corefiles:
+            for core in corefiles:
+                core.unlink()
+    
+    def check_quals(self):
+
+        quals = ['mash', 'seqtk']
+        for q in quals:
+            self.check_installation(software = q)
+
+
+    def check_sa(self):
+        
+        self.check_quals()
+        self.check_snippycore()
+        self.check_snpdists()
+        self.check_kraken2DB()
+        self.check_iqtree()
+        self.check_assembler()
+        self.check_assemble_accesories()
+
+        return(self.check_snippy())
 
     def check_deps(self):
         '''
@@ -302,32 +380,30 @@ class RunSnpDetection(object):
         # TODO check all software tools used and is there a way to check database last update??
         # TODO check assemblers
         self.logger.info(f"Checking software dependencies")
-        if self.pipeline != "a":
-            self.check_snippycore()
-            self.check_snpdists()
-            self.check_kraken2DB()
-            self.check_iqtree()
-            return(self.check_snippy())
-        elif self.pipeline != "s":
-            self.check_assembler()
-            self.check_assemble_accesories()
+        # self.logger.info(f"{self.pipeline}")
+        if self.pipeline in ["sa", "preview"]:
+            return(self.check_sa())
+
+        if self.pipeline == "all":
             self.check_roary()
-            return True
+            return(self.check_sa())
+            
     
     def run_checks(self):
         '''
         Run checks prior to start - checking all software is installed, if this is a rerun and the input files
         '''
         self.check_setup_files()
-        
-        self.check_deps()
+        # self.logger.info(f"{self.ref}")
+        # self.logger.info(f"Checking software dependencies")
+        self.snippy_version = self.check_deps()
         # check reference
-        if self.pipeline != 'a':
-            if self.ref == '':
-                self.logger.warning(f"You are trying call SNPs, a reference file is required. Please try again using '-r path to reference'")
-                raise SystemExit
-            else:
-                self.ref = self.link_file(self.ref)
+        if self.ref == '':
+            self.logger.warning(f"You are trying call SNPs, a reference file is required. Please try again using '-r path to reference'")
+            raise SystemExit
+        else:
+            self.logger.info(f"Starting to link reference")
+            self.ref = self.link_file(self.ref)
         
     def set_cluster_log(self):
         '''
@@ -348,13 +424,13 @@ class RunSnpDetection(object):
 
     def set_source_log(self):
         '''
-        set the reference, mask and id for tracking and potential.
-        
-            
+        set the reference, mask and id for tracking and potential.           
         '''   
+        
         # TODO add in options for using singularity containers
         # path if using containers.
-        snippy_v = f'singularity_{self.day}' if self.use_singularity else self.snippy_version
+        snippy_v = f'singularity_{self.day}' if self.use_singularity else self.snippy_version.group()
+        self.logger.info(f"Snippy : {snippy_v} has been added to the job log file.")
         kraken = self.kraken_db if self.run_kraken else ''
         s = True if self.use_singularity else False
         self.logger.info(f"Recording your settings for job: {self.job_id}")
@@ -363,12 +439,13 @@ class RunSnpDetection(object):
                                     'Date':self.day, 'User':self.user, 'snippy_version':snippy_v, 'input_file':f"{self.input_file}",'prefillpath': self.prefillpath, 'cluster': self.cluster,'singularity': s, 'kraken_db':kraken, 'Gubbins': self.gubbins}, 
                                     index=[0], )
         
-        source_path = self.workdir / 'source.log'
+        source_path = self.workdir / 'job.log'
         if source_path.exists():
             source_df = pandas.read_csv(source_path, '\t')
             source_df = source_df.append(new_df, sort = True)
         else:
             source_df = new_df
+
         
         source_df.to_csv(source_path , index=False, sep = '\t')
     
@@ -391,10 +468,10 @@ class RunSnpDetection(object):
         if isinstance(report_path, str):
             report_path = pathlib.Path(report_path)
         if report_path.exists() and not preview_path.exists():
-            self.logger.warning(f"This may be a re-run of an existing job. Please try again using rerun instead of run OR use -f to force an overwrite of the existing job.")
-            self.logger.warning(f"Exiting....")
-            
-            raise SystemExit()
+            self.logger.info(f"This appears to be a rerun of an existing job. Previous result will be removed unless you use --keep.")  
+            self.rerun_report()
+            self.remove_core()        
+            return True
         elif preview_path.exists():
             self.setup_for_rerun()
             return False
@@ -500,8 +577,10 @@ class RunSnpDetection(object):
         output:
             returns path.name (str)   
         '''
-        
-        self.logger.info(f"Getting input files.") 
+        J = self.workdir / self.job_id
+        if not J.exists():
+            J.mkdir()
+        self.logger.info(f"Getting input files from {path}.") 
         if path.exists():
             if f"{path.suffix}" in ['.gz','zip']:
                     path = pathlib.Path(self.unzip_files(path, f"{path.suffix}"))
@@ -509,12 +588,14 @@ class RunSnpDetection(object):
                         self.logger.warning(f"{path} does not exist. Please try again.")
                         raise SystemExit
             else:
-                target = self.workdir / path.name
+                target = J /  path.name
+                # self.logger.info(f"Target is {target}")
                 # use rename to copy reference to working directory
                 # if the reference is not already in the working directory symlink it to working dir
                 if not target.exists():
-                    self.logger.info(f"Linking {path.name} to {self.workdir.name}")
-                    target.symlink_to(path)
+                    self.logger.info(f"Copying {path.name} to {J}")
+                    # target.symlink_to(path)
+                    subprocess.run(f"cp {path} {target}", shell = True)
                     found = True
                     
         else:
@@ -524,6 +605,22 @@ class RunSnpDetection(object):
         
         return  path.name
 
+    def index_reference(self):
+
+        ref = pathlib.Path(self.job_id , 'ref.fa')
+        idx = pathlib.Path(self.job_id , 'ref.fa.fai')
+        if '.fa' not in self.ref:
+            self.logger.info(f"converting {self.ref} to fasta format.")
+            SeqIO.convert(f"{self.ref}", 'genbank', f"{ref}", 'fasta')
+        else:
+            subprocess.run(f"cp {self.ref} {ref}", shell = True)
+        record = SeqIO.parse(f"{ref}", "fasta")
+        for r in record:
+            r.description = ''
+            SeqIO.write(r, f"{ref}", "fasta")
+        self.logger.info(f"Indexing reference.")
+        subprocess.run(f"samtools faidx {ref}", shell =True)
+
     def check_mask(self, mask, original_mask = False):
         '''
         input:
@@ -663,35 +760,7 @@ class RunSnpDetection(object):
         isolates = self.set_isolate_log(tab = tab, logfile = logfile, validation = validation)
         
         self.logger.info(f"This job : {self.job_id} contains {len(list(set(isolates)))}")
-        return(list(set(isolates))) 
-    
-    
-    def write_pipeline_job(self):
-        '''
-        write out the pipeline string for transfer to job specific pipeline
-        '''
-        
-        wd = self.workdir / self.job_id
-        # pipeline_setup = {
-        #     's':'Snakefile_snippy',
-        #     'sa':'Snakefile_default_',
-        #     'a':'Snakefile_assembly',
-        #     'all': 'Snakefile_all'
-        # }
-        vars_for_file = {
-            'workdir': f"{wd}",
-            'singularity_dir' : self.singularity_path, 
-        }
-        
-        self.logger.info(f"Writing Snakefile for job : {self.job_id}")
-        snk_template = jinja2.Template(pathlib.Path(self.resources, "bohra_v2.smk").read_text())
-        snk = self.workdir / self.job_id/ 'Snakefile'
-
-        snk.write_text(snk_template.render(vars_for_file)) 
-        
-        self.logger.info(f"Snakefile successfully created")
-        
-        
+        return(list(set(isolates)))         
 
     def json_setup(self, queue_args):
         '''
@@ -721,7 +790,8 @@ class RunSnpDetection(object):
 
 
     def cluster_cmd(self):
-
+        snake_name = f"{pathlib.Path(__file__).parent / 'utils'/ 'bohra.smk'}"
+        wd = f"{self.workdir / self.job_id}"
         queue_args = ""
         self.logger.info(f"Setting up cluster settings for {self.job_id} using {self.json}")
         if self.queue == 'sbatch':
@@ -738,11 +808,9 @@ class RunSnpDetection(object):
 
         return f"snakemake -j 999 --cluster-config {self.json} --cluster '{queue_cmd} {queue_string}'"
 
-        
-
-
     
-    def setup_workflow(self, isolates, config_name = 'config.yaml', snake_name = 'Snakefile'):
+    
+    def setup_workflow(self, isolates, config_name = 'config.yaml'):
         '''
         generate job specific snakefile and config.yaml
         input:
@@ -751,16 +819,15 @@ class RunSnpDetection(object):
 
         self.logger.info(f"Setting up {self.job_id} specific workflow")
         
-        gubbins_string = ""
         # make a masking string
         wd = self.workdir / self.job_id
         if self.mask != '':
-            maskstring = f"{self.workdir / self.mask}"
+            maskstring = f"{self.workdir / self.job_id /self.mask}"
         else:
             maskstring = ''
         self.logger.info(f'Mask string : {maskstring}')
         self.logger.info(f"Writing config file for job : {self.job_id}")
-        
+        reference = pathlib.Path(self.ref)
         vars_for_file = {
             'workdir': f"{wd}",
             'prefill_path' : self.prefillpath,
@@ -769,7 +836,7 @@ class RunSnpDetection(object):
             'mask_string': maskstring, 
             'template_path':f"{pathlib.Path(__file__).parent / 'templates'}",
             'script_path':f"{pathlib.Path(__file__).parent / 'utils'}",
-            'reference' : f"{pathlib.Path(self.workdir, self.ref)}",
+            'reference' : f"{wd / reference.name}",
             'minperc' : self.minaln,
             'now' : self.now,
             'day': self.day, 
@@ -779,7 +846,9 @@ class RunSnpDetection(object):
             'min_cov': self.mincov,
             'kraken_db': f"{self.kraken_db}",
             'preview': self.preview, 
-            'prefill_path': self.prefillpath if self.prefillpath != '' else 'nopath'
+            'prefill_path': self.prefillpath if self.prefillpath != '' else 'nopath',
+            # 'snippy_singularity': self.snippy_singularity,
+            # 'abritamr_singularity': self.abritamr_singularity
         }
 
         # read the config file which is written with jinja2 placeholders (like django template language)
@@ -789,19 +858,16 @@ class RunSnpDetection(object):
         config.write_text(config_template.render(vars_for_file))
         
         self.logger.info(f"Config file successfully created")
-
-        self.write_pipeline_job()
-        
-
- 
-    def run_workflow(self,snake_name = 'Snakefile'):
+       
+    def run_workflow(self):
         '''
         run snp_detection
         set the current directory to working dir for correct running of pipeline
         if the pipeline works, return True else False
         '''
+        snake_name = f"{pathlib.Path(__file__).parent / 'utils'/ 'bohra.smk'}"
         if self.use_singularity:
-            singularity_string = f"--use-singularity --singularity-args '--bind /home'"
+            singularity_string = f"--use-singularity --singularity-args \"--home /home/khhor\""
         else:
             singularity_string = ''
 
@@ -815,19 +881,23 @@ class RunSnpDetection(object):
             dry = '-np'
         else:
             dry = ''
-
+        wd = self.workdir / self.job_id
         if self.cluster:
-            cmd = f"{self.cluster_cmd()} -s {snake_name} {force} {singularity_string} --latency-wait 1200"
+            cmd = f"{self.cluster_cmd()} -s {snake_name} -d {wd} {force} {singularity_string} --latency-wait 1200"
         else:
-            cmd = f"cd {self.job_id} && snakemake {dry} -s {snake_name} -j {self.cpus} {force} {singularity_string}"
+            cmd = f"snakemake {dry} -s {snake_name} {singularity_string} -j {self.cpus} -d {self.job_id} {force} --verbose 2>&1"
             # cmd = f"snakemake -s {snake_name} --cores {self.cpus} {force} "
         self.logger.info(f"Running job : {self.job_id} with {cmd} this may take some time. We appreciate your patience.")
         wkf = subprocess.run(cmd, shell = True)
-        # while True:
-        #     line = wkf.stdout.readline().strip()
-        #     if not line:
-        #         break
-        #     self.self.logger.info(f"{line}")
+        
+        while True:
+            if wkf.stdout != None:
+                line = wkf.stdout.readline().strip()
+                if not line:
+                    break
+            line = ''
+            break
+            self.self.logger.info(f"{line}")
         if wkf.returncode == 0:
             return True
         else:
@@ -847,7 +917,7 @@ class RunSnpDetection(object):
             self.logger.info(f"You have chosen to run bohra with singularity containers. Good luck")
         else:
             self.run_checks()
-        
+        self.index_reference()
         # update source data in source.log
         self.set_source_log()
         
@@ -868,7 +938,8 @@ class RunSnpDetection(object):
                     force = f"-F"
                 else:
                     force = f""
-                self.logger.info(f"snakemake -j {self.jobs} {force} 2>&1 | tee -a bohra.log")
+                # self.logger.info(f"snakemake -j {self.jobs} {force} 2>&1 ")
             self.logger.info(f"Have a nice day. Come back soon.") 
-            
+
+        
 
